/***********************************************************************
 * A SystemVerilog model of a  4-bit counter and 20-bit LFSR
 *
 * The purpose of the lab is to experiment with the reg, logic and bit
 * data types
 **********************************************************************/

module cntr_lfsr
 #(parameter type DTYPE = logic)   // module uses redefinable data type
  (input  DTYPE        clk, reset_n,
   output DTYPE [19:0] lfsr_R,
   output DTYPE [ 3:0] cnt16_R
  );
  timeunit 1ns;
  timeprecision 1ns;

  initial begin
     $display("\n***** cntr_lfsr was compiled using %s types *****\n", $typename(DTYPE));
  end

  always_ff @(posedge clk or negedge reset_n)
    if (!reset_n) begin
      cnt16_R <= 0;
      lfsr_R  <= 1;  //must be reset to a non zero value
    end
    else
    begin
      cnt16_R    <= cnt16_R + 1;
      lfsr_R     <= {lfsr_R[0] , lfsr_R[19:1]};
      lfsr_R[2]  <= lfsr_R[3]  ^ lfsr_R[0];
      lfsr_R[3]  <= lfsr_R[4]  ^ lfsr_R[0];
      lfsr_R[18] <= lfsr_R[19] ^ lfsr_R[0];
    end

endmodule
