/***********************************************************************
 * Testbench for the counter/LFSR RTL model.
 **********************************************************************/
module test
 #(parameter type DTYPE = logic)   // module sues redefinable data type
  (output DTYPE        clk, reset_n,
   input  DTYPE [19:0] lfsr_R,
   input  DTYPE [ 3:0] cnt16_R
  );
  timeunit 1ns;
  timeprecision 1ns;

  initial begin
     $display("\n***** test module was compiled using %s types *****", $typename(DTYPE));
  end

  initial begin  // let design run several clock cycles
    clk = 0;
    repeat(36) #5ns clk = ~clk;
//    $stop();
    $finish();
  end

  initial begin  // assert reset
    #1ns  reset_n = 0;
    #18ns reset_n = 1;  //reset is released away from a clk edge
  end

  always @(posedge clk)  //check the LFSR output for unknown bits (x or z)
    #0ns if (^lfsr_R === 1'bx)
      $display("ERROR! at %t, lfsr_R has at least 1 bit at X or Z", $time);

  initial begin  // print dut outputs whenever they change
    $timeformat(-9, 0, " ns", 8);
    $monitor("At %t: cnt16_R = %h, lfsr_R = %b", $time, cnt16_R, lfsr_R);
    end

endmodule
