/***********************************************************************
 * An interface for connecting two simple processors together.
 *
 **********************************************************************/


interface proc_ifc
  import type_defs_pkg::*;
  (input logic clk, rst);
  timeunit 1ns; timeprecision 1ns;

  bit       req, done, ack;
  packet_t  data_master, data_slave;

  function logic parity_gen(packet_t data);
    return(^{data.opA,data.opB,data.opcode});
  endfunction

  task parity_check(packet_t data);
      if (data.parity !== ^{data.opA,data.opB,data.opcode})
        $display("ERROR! Parity error in packet_t");
  endtask

  modport bus_master (output req, ack,
                      output data_master,
                      input  data_slave,
                      input  done,
                      input  clk, rst,
                      import parity_check, parity_gen
                     );

  modport bus_slave  (input  req, ack,
                      input  data_master,
                      output data_slave,
                      output done,
                      input  clk, rst,
                      import parity_check, parity_gen
                     );

endinterface: proc_ifc
