
class BaseTest;

  Driver drv_h;
  Scoreboard scb_h;
  Monitor mon_h;
  Generator gen_h;

  mailbox #(Transaction) mon2scb_h;
  mailbox #(Transaction) gen2scb_h;
  // ***LAB INSERT  add  the gen2drv mailbox
  mailbox #(Transaction) gen2drv_h;

  task build();
    scb_h = new();
    drv_h = new(instr_register_test_pkg::v_io);
    mon_h = new(instr_register_test_pkg::v_io);
    // ***LAB INSERT  remove argument from the constructor for Generator 
    gen_h = new();
    mon2scb_h = new();  // unbounded so that no monitored transactions are missed.
    gen2scb_h = new();  // unbounded so that the transactions can be extracted as the monitored ones arrive.
    // ***LAB INSERT  create the gen2drv mailbox 
    gen2drv_h = new(1);
   // call the build of any children
    scb_h.build();
    drv_h.build();
    gen_h.build();
    mon_h.build();
  endtask


  // ***LAB INSERT  put the call to the connect class around the connect code. 
  task connect();
    // connecting up the handles defined in the child classes to the mailboxes
    mon_h.mon2scb_h = mon2scb_h;
    scb_h.mon2scb_h = mon2scb_h;
    scb_h.gen2scb_h = gen2scb_h;
    gen_h.gen2scb_h = gen2scb_h;
    // ***LAB INSERT  create the gen2drv mailbox to the handles in gen_h and drv_h 
    gen_h.gen2drv_h = gen2drv_h;
    drv_h.gen2drv_h = gen2drv_h;

    // call the connect of any children
    scb_h.connect();
    drv_h.connect();
    gen_h.connect();
    mon_h.connect();
  endtask : connect


  task reset();
    $display("\nReseting the instruction register...");
    fork 
      scb_h.reset();
      drv_h.reset();
      gen_h.reset();
      mon_h.reset();
    join
  endtask : reset

  task run(int n);
    fork
      scb_h.run(n); 
      drv_h.run(n); 
      gen_h.run(n); 
      mon_h.run(n); 
    join
  endtask

  task check(int n);
    // read back and display same three register locations
    $display("\nChecking the read values against the write values....");
    fork    
      scb_h.check(n);
      drv_h.check(n);
      gen_h.check(n);
      mon_h.check(n);
    join
  endtask

  task report();
    scb_h.report();
    drv_h.report();
    gen_h.report();
    mon_h.report();
  endtask : report

endclass: BaseTest